@@ -1,5 +1,6 @@
 import asyncio
-from datetime import datetime, timezone
+from contextlib import AsyncExitStack
+from datetime import UTC, datetime
 import logging
 import math
 from unittest.mock import call
@@ -8,6 +9,7 @@ import pytest
 
 from zigpy import device, endpoint
 import zigpy.application
+from zigpy.datastructures import RequestLimiter
 import zigpy.exceptions
 from zigpy.ota import OtaImagesResult
 import zigpy.ota.image
@@ -188,7 +190,7 @@ async def test_broadcast(app_mock):
 async def _get_node_descriptor(dev, zdo_success=True, request_success=True):
     async def mockrequest(nwk, tries=None, delay=None, **kwargs):
         if not request_success:
-            raise asyncio.TimeoutError
+            raise TimeoutError
 
         status = 0 if zdo_success else 1
         return [status, nwk, zdo_t.NodeDescriptor.deserialize(b"abcdefghijklm")[0]]
@@ -307,13 +309,13 @@ def test_device_last_seen(dev, monkeypatch):
     assert dev.last_seen is None
 
     dev.last_seen = 0
-    epoch = datetime(1970, 1, 1, 0, 0, 0, 0, tzinfo=timezone.utc)
+    epoch = datetime(1970, 1, 1, 0, 0, 0, 0, tzinfo=UTC)
     assert dev.last_seen == epoch.timestamp()
 
     dev.listener_event.assert_called_once_with("device_last_seen_updated", epoch)
     dev.listener_event.reset_mock()
 
-    now = datetime.now(timezone.utc)
+    now = datetime.now(UTC)
     dev.last_seen = now
     dev.listener_event.assert_called_once_with("device_last_seen_updated", now)
 
@@ -1278,7 +1280,9 @@ async def test_debouncing(dev):
 
 async def test_device_concurrency(dev: device.Device) -> None:
     """Test that the device can handle multiple requests concurrently."""
-    dev._concurrent_requests_semaphore.max_value = 1
+    dev._concurrent_requests_semaphore = RequestLimiter(
+        max_concurrency=1, capacities={t.PacketPriority.LOW: 1}
+    )
 
     ep = dev.add_endpoint(1)
     ep.add_input_cluster(Basic.cluster_id)
@@ -1360,10 +1364,10 @@ async def test_duplicate_request_sending(dev: device.Device) -> None:
 
     async def delayed_receive(*args, **kwargs) -> None:
         await asyncio.sleep(0.1)
-        raise asyncio.TimeoutError()
+        raise TimeoutError()
 
     dev._application.request = AsyncMock(side_effect=delayed_receive)
-    dev._concurrent_requests_semaphore.max_value = 100000
+    dev._concurrent_requests_semaphore.max_concurrency = 100000
 
     # We send 256 + 1 requests
     errors = await asyncio.gather(
@@ -1471,7 +1475,9 @@ async def test_poll_control_checkin_callback(
     expected_fast_poll: bool,
 ) -> None:
     """Test PollControl check-in callback with different device states."""
-    dev._concurrent_requests_semaphore.max_value = 1
+    dev._concurrent_requests_semaphore = RequestLimiter(
+        max_concurrency=1, capacities={t.PacketPriority.LOW: 1}
+    )
 
     ep = dev.add_endpoint(1)
     poll_control = ep.add_input_cluster(PollControl.cluster_id)
@@ -1486,46 +1492,45 @@ async def test_poll_control_checkin_callback(
     else:
         dev._initialize_task = None
 
-    if semaphore_locked:
-        await dev._concurrent_requests_semaphore.acquire()
-
-    zcl_hdr = foundation.ZCLHeader(
-        frame_control=foundation.FrameControl(
-            frame_type=foundation.FrameType.CLUSTER_COMMAND,
-            is_manufacturer_specific=False,
-            direction=foundation.Direction.Server_to_Client,
-            disable_default_response=1,
-            reserved=0,
-        ),
-        tsn=0x12,
-        command_id=PollControl.ClientCommandDefs.checkin.id,
-    )
-    command = PollControl.ClientCommandDefs.checkin.schema()
-
-    # Test the callback
-    await dev.poll_control_checkin_callback(zcl_hdr, command)
-
-    # Verify the correct response was sent
-    if expected_fast_poll:
-        assert poll_control.checkin_response.mock_calls == [
-            call(
-                start_fast_polling=expected_fast_poll,
-                fast_poll_timeout=int(device.DEFAULT_FAST_POLL_TIMEOUT * 4),
-                tsn=0x12,
+    async with AsyncExitStack() as stack:
+        if semaphore_locked:
+            await stack.enter_async_context(
+                dev._concurrent_requests_semaphore(priority=t.PacketPriority.LOW)
             )
-        ]
-    else:
-        assert poll_control.checkin_response.mock_calls == [
-            call(
-                start_fast_polling=expected_fast_poll,
-                fast_poll_timeout=0,
-                tsn=0x12,
-            )
-        ]
 
-    # Clean up semaphore if we acquired it
-    if semaphore_locked:
-        dev._concurrent_requests_semaphore.release()
+        zcl_hdr = foundation.ZCLHeader(
+            frame_control=foundation.FrameControl(
+                frame_type=foundation.FrameType.CLUSTER_COMMAND,
+                is_manufacturer_specific=False,
+                direction=foundation.Direction.Server_to_Client,
+                disable_default_response=1,
+                reserved=0,
+            ),
+            tsn=0x12,
+            command_id=PollControl.ClientCommandDefs.checkin.id,
+        )
+        command = PollControl.ClientCommandDefs.checkin.schema()
+
+        # Test the callback
+        await dev.poll_control_checkin_callback(zcl_hdr, command)
+
+        # Verify the correct response was sent
+        if expected_fast_poll:
+            assert poll_control.checkin_response.mock_calls == [
+                call(
+                    start_fast_polling=expected_fast_poll,
+                    fast_poll_timeout=int(device.DEFAULT_FAST_POLL_TIMEOUT * 4),
+                    tsn=0x12,
+                )
+            ]
+        else:
+            assert poll_control.checkin_response.mock_calls == [
+                call(
+                    start_fast_polling=expected_fast_poll,
+                    fast_poll_timeout=0,
+                    tsn=0x12,
+                )
+            ]
 
 
 async def test_begin_fast_polling_with_cluster(dev: device.Device) -> None:
@@ -1614,7 +1619,7 @@ async def test_initialize_fast_polling_failure(dev: device.Device) -> None:
     ep = dev.add_endpoint(1)
     ep.add_input_cluster(PollControl.cluster_id)
 
-    dev.begin_fast_polling = AsyncMock(side_effect=[asyncio.TimeoutError(), None])
+    dev.begin_fast_polling = AsyncMock(side_effect=[TimeoutError(), None])
 
     async def mockepinit(self, *args, **kwargs):
         self.status = endpoint.Status.ZDO_INIT
@@ -1637,19 +1642,53 @@ async def test_initialize_fast_polling_failure(dev: device.Device) -> None:
     assert dev.begin_fast_polling.mock_calls == [call()]
 
 
-async def test_device_flipped_cluster_warning(dev: device.Device, caplog) -> None:
-    """Test that a warning is logged when a cluster is flipped."""
-    ep1 = dev.add_endpoint(1)
-    ep1.add_input_cluster(OnOff.cluster_id)
+@pytest.mark.parametrize(
+    (
+        "has_input_cluster",
+        "has_output_cluster",
+        "packet_direction",
+        "expected_cluster_type",
+    ),
+    [
+        # Correct cluster matching
+        (True, False, foundation.Direction.Server_to_Client, ClusterType.Server),
+        (False, True, foundation.Direction.Client_to_Server, ClusterType.Client),
+        # Direction flipping: only one cluster type exists, packet has wrong direction
+        (True, False, foundation.Direction.Client_to_Server, ClusterType.Server),
+        (False, True, foundation.Direction.Server_to_Client, ClusterType.Client),
+        # Both clusters exist: should match based on direction, no flipping
+        (True, True, foundation.Direction.Server_to_Client, ClusterType.Server),
+        (True, True, foundation.Direction.Client_to_Server, ClusterType.Client),
+        # Cluster doesn't exist
+        (False, False, foundation.Direction.Server_to_Client, None),
+        (False, False, foundation.Direction.Client_to_Server, None),
+    ],
+)
+async def test_device_cluster_direction_flipping(
+    dev: device.Device,
+    has_input_cluster: bool,
+    has_output_cluster: bool,
+    packet_direction: foundation.Direction,
+    expected_cluster_type: ClusterType | None,
+) -> None:
+    """Test that cluster direction flipping routes messages to the correct cluster."""
+    ep = dev.add_endpoint(1)
 
-    ep2 = dev.add_endpoint(2)
-    ep2.add_output_cluster(OnOff.cluster_id)
+    if has_input_cluster:
+        input_cluster = ep.add_input_cluster(OnOff.cluster_id)
+    else:
+        input_cluster = None
+
+    if has_output_cluster:
+        output_cluster = ep.add_output_cluster(OnOff.cluster_id)
+    else:
+        output_cluster = None
 
     zcl_hdr = foundation.ZCLHeader(
         frame_control=foundation.FrameControl(
             frame_type=foundation.FrameType.CLUSTER_COMMAND,
             is_manufacturer_specific=False,
-            direction=foundation.Direction.Client_to_Server,
+            direction=packet_direction,
             disable_default_response=1,
             reserved=0,
         ),
@@ -1671,14 +1710,30 @@ async def test_device_flipped_cluster_warning(dev: device.Device, caplog) -> Non
         rssi=-30,
     )
 
-    # Correct
-    with caplog.at_level(logging.WARNING):
-        dev.packet_received(packet.replace(src_ep=2))
+    captured_result = []
 
-    assert "has incorrect direction" not in caplog.text
+    original_match = dev._match_packet_endpoint_cluster
 
-    # Incorrect
-    with caplog.at_level(logging.WARNING):
-        dev.packet_received(packet.replace(src_ep=1))
+    def capture_match(*args, **kwargs):
+        result = original_match(*args, **kwargs)
+        captured_result.append(result)
+        return result
 
-    assert "has incorrect direction" in caplog.text
+    with patch.object(
+        dev, "_match_packet_endpoint_cluster", side_effect=capture_match
+    ) as spy:
+        dev.packet_received(packet)
+
+    assert spy.call_count == 1
+    assert len(captured_result) == 1
+
+    _, returned_cluster = captured_result[0]
+
+    if expected_cluster_type is None:
+        assert returned_cluster is None
+    elif expected_cluster_type is ClusterType.Server:
+        assert returned_cluster is input_cluster
+    elif expected_cluster_type is ClusterType.Client:
+        assert returned_cluster is output_cluster
+    else:
+        pytest.fail("Unexpected cluster type")
