@@ -1,9 +1,10 @@
 """Tests for the quirks v2 module."""
 
 import pathlib
-from typing import Final
+from typing import Any, Final
 from unittest.mock import AsyncMock
 
+from frozendict import frozendict
 import pytest
 
 from zigpy.const import (
@@ -36,6 +37,7 @@ from zigpy.quirks.v2 import (
     ZCLCommandButtonMetadata,
     ZCLSensorMetadata,
     add_to_registry_v2,
+    recursive_freeze,
 )
 from zigpy.quirks.v2.homeassistant import EntityType, UnitOfTime
 from zigpy.quirks.v2.homeassistant.sensor import SensorDeviceClass, SensorStateClass
@@ -1014,12 +1016,12 @@ async def test_quirks_v2_matches_v1(app_mock):
     triggers = {
         (SHORT_PRESS, TURN_ON): {
             COMMAND: COMMAND_TOGGLE,
-            CLUSTER_ID: 6,
+            CLUSTER_ID: OnOff.cluster_id,
             ENDPOINT_ID: 1,
         },
         (LONG_PRESS, TURN_ON): {
             COMMAND: COMMAND_RELEASE,
-            CLUSTER_ID: 5,
+            CLUSTER_ID: Scenes.cluster_id,
             ENDPOINT_ID: 1,
             PARAMS: {"param1": 0},
         },
@@ -1153,6 +1155,12 @@ async def test_quirks_v2_matches_v1(app_mock):
         )
 
     assert quirked.device_automation_triggers == quirked_v2.device_automation_triggers
+    assert (
+        quirked.device_automation_triggers[("remote_button_long_press", "turn_on")][
+            "cluster_id"
+        ]
+        == 0x0005
+    )
 
 
 async def test_quirks_v2_add_to_registry_v2_logs_error(caplog):
@@ -1445,6 +1453,7 @@ async def test_quirks_v2_change_entity_metadata(device_mock: Device) -> None:
             endpoint_id=1,
             cluster_id=OnOff.cluster_id,
             new_translation_key="custom_key",
+            new_translation_placeholders={"index": "subkey"},
         )
         .change_entity_metadata(
             endpoint_id=1,
@@ -1473,6 +1482,7 @@ async def test_quirks_v2_change_entity_metadata(device_mock: Device) -> None:
             new_primary=True,
             new_unique_id=None,
             new_translation_key=None,
+            new_translation_placeholders=None,
             new_device_class=None,
             new_state_class=None,
             new_entity_category=None,
@@ -1488,6 +1498,7 @@ async def test_quirks_v2_change_entity_metadata(device_mock: Device) -> None:
             new_primary=None,
             new_unique_id=None,
             new_translation_key="custom_key",
+            new_translation_placeholders={"index": "subkey"},
             new_device_class=None,
             new_state_class=None,
             new_entity_category=None,
@@ -1503,6 +1514,7 @@ async def test_quirks_v2_change_entity_metadata(device_mock: Device) -> None:
             new_primary=None,
             new_unique_id=None,
             new_translation_key=None,
+            new_translation_placeholders=None,
             new_device_class=SensorDeviceClass.POWER,
             new_state_class=SensorStateClass.MEASUREMENT,
             new_entity_category=EntityType.CONFIG,
@@ -1518,6 +1530,7 @@ async def test_quirks_v2_change_entity_metadata(device_mock: Device) -> None:
             new_primary=None,
             new_unique_id="custom_unique_id",
             new_translation_key=None,
+            new_translation_placeholders=None,
             new_device_class=None,
             new_state_class=None,
             new_entity_category=None,
@@ -1525,3 +1538,65 @@ async def test_quirks_v2_change_entity_metadata(device_mock: Device) -> None:
             new_fallback_name="Custom Fallback Name",
         ),
     )
+
+
+def strict_eq(a: Any, b: Any) -> bool:
+    """Recursively check equality and type matching."""
+    if type(a) is not type(b):
+        return False
+    if a != b:
+        return False
+    if isinstance(a, dict):
+        if a.keys() != b.keys():
+            return False
+        return all(strict_eq(a[k], b[k]) for k in a)
+    if isinstance(a, tuple | list):
+        if len(a) != len(b):
+            return False
+        return all(
+            strict_eq(a_item, b_item) for a_item, b_item in zip(a, b, strict=False)
+        )
+    return True
+
+
+@pytest.mark.parametrize(
+    ("obj", "expected"),
+    [
+        ({}, frozendict()),
+        ([], ()),
+        ({"a": 1, "b": 2}, frozendict({"a": 1, "b": 2})),
+        ([1, 2, 3], (1, 2, 3)),
+        (
+            {"outer": {"inner": "value"}},
+            frozendict({"outer": frozendict({"inner": "value"})}),
+        ),
+        ({"key": [1, 2, 3]}, frozendict({"key": (1, 2, 3)})),
+        ([{"a": 1}, {"b": 2}], (frozendict({"a": 1}), frozendict({"b": 2}))),
+        ([[1, 2], [3, 4]], ((1, 2), (3, 4))),
+        (
+            {"a": [1, {"b": 2}], "c": {"d": [3, 4]}},
+            frozendict(
+                {"a": (1, frozendict({"b": 2})), "c": frozendict({"d": (3, 4)})}
+            ),
+        ),
+        (42, 42),
+        ("string", "string"),
+        (None, None),
+        (True, True),
+        (3.14, 3.14),
+        (frozendict({"a": 1}), frozendict({"a": 1})),
+        ((1, 2, 3), (1, 2, 3)),
+        ((1, [2], {3: 4}), (1, (2,), frozendict({3: 4}))),
+        (
+            {"triggers": {("key1", "key2"): {"param": 0}}},
+            frozendict(
+                {"triggers": frozendict({("key1", "key2"): frozendict({"param": 0})})}
+            ),
+        ),
+    ],
+)
+def test_recursive_freeze(obj, expected):
+    """Test recursive_freeze converts mutable collections to immutable ones."""
+    result = recursive_freeze(obj)
+    assert strict_eq(result, expected)
+    hash(result)
