@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import asyncio
 import contextlib
-from datetime import datetime, timedelta, timezone
+from datetime import UTC, datetime, timedelta
 import json
 import logging
 import re
@@ -23,16 +23,17 @@ import zigpy.state
 import zigpy.types as t
 import zigpy.typing
 import zigpy.util
+from zigpy.zcl import ClusterType
 from zigpy.zcl.clusters.general import Basic
 from zigpy.zdo import types as zdo_t
 
 LOGGER = logging.getLogger(__name__)
 
-DB_VERSION = 12
+DB_VERSION = 13
 DB_V = f"_v{DB_VERSION}"
 MIN_SQLITE_VERSION = (3, 24, 0)
 
-UNIX_EPOCH = datetime.fromtimestamp(0, tz=timezone.utc)
+UNIX_EPOCH = datetime.fromtimestamp(0, tz=UTC)
 DB_V_REGEX = re.compile(r"(?:_v\d+)?$")
 
 MIN_UPDATE_DELTA = timedelta(seconds=30).total_seconds()
@@ -56,14 +57,13 @@ def _import_compatible_sqlite3(min_version: tuple[int, int, int]) -> types.Modul
 
         if module.sqlite_version_info >= min_version:
             return module
-    else:
-        min_ver = ".".join(map(str, min_version))
+    min_ver = ".".join(map(str, min_version))
 
-        raise RuntimeError(
-            f"zigpy requires SQLite {min_ver} or newer. If your distribution does not"
-            f" provide a more recent release, install pysqlite3 with"
-            f" `pip install pysqlite3-binary`"
-        )
+    raise RuntimeError(
+        f"zigpy requires SQLite {min_ver} or newer. If your distribution does not"
+        f" provide a more recent release, install pysqlite3 with"
+        f" `pip install pysqlite3-binary`"
+    )
 
 
 sqlite3 = _import_compatible_sqlite3(min_version=MIN_SQLITE_VERSION)
@@ -122,7 +122,17 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             status = "\n".join(row[0] for row in rows)
 
             if status != "ok":
-                LOGGER.error("SQLite database file is corrupted!\n%s", status)
+                LOGGER.error(
+                    "Zigbee database is corrupted, integrity check failed!\n%s", status
+                )
+
+        async with self.execute("PRAGMA foreign_key_check") as cursor:
+            rows = await cursor.fetchall()
+
+            if rows:
+                LOGGER.error(
+                    "Zigbee database is corrupted, foreign key check failed!\n%s", rows
+                )
 
         # Truncate the SQLite journal file instead of deleting it after transactions
         await self._set_isolation_level(None)
@@ -148,7 +158,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
 
         try:
             await listener.initialize_tables()
-        except Exception:
+        except Exception:  # noqa: BLE001
             await listener.shutdown()
             raise
 
@@ -170,7 +180,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                     args,
                     str(exc),
                 )
-            except Exception as ex:
+            except Exception as ex:  # noqa: BLE001
                 LOGGER.error(
                     "Unexpected error while processing %s(%s): %s", cb_name, args, ex
                 )
@@ -278,12 +288,23 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             "_save_attribute",
             cluster.endpoint.device.ieee,
             cluster.endpoint.endpoint_id,
+            cluster.cluster_type,
             cluster.cluster_id,
             attrid,
             value,
             timestamp,
         )
 
+    def attribute_cleared(self, cluster: zigpy.typing.ClusterType, attrid: int) -> None:
+        self.enqueue(
+            "_clear_attribute",
+            cluster.endpoint.device.ieee,
+            cluster.endpoint.endpoint_id,
+            cluster.cluster_type,
+            cluster.cluster_id,
+            attrid,
+        )
+
     def unsupported_attribute_added(
         self, cluster: zigpy.typing.ClusterType, attrid: int
     ) -> None:
@@ -291,17 +312,23 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             "_unsupported_attribute_added",
             cluster.endpoint.device.ieee,
             cluster.endpoint.endpoint_id,
+            cluster.cluster_type,
             cluster.cluster_id,
             attrid,
         )
 
     async def _unsupported_attribute_added(
-        self, ieee: t.EUI64, endpoint_id: int, cluster_id: int, attrid: int
+        self,
+        ieee: t.EUI64,
+        endpoint_id: int,
+        cluster_type: ClusterType,
+        cluster_id: int,
+        attrid: int,
     ) -> None:
-        q = f"""INSERT INTO unsupported_attributes{DB_V} VALUES (?, ?, ?, ?)
-                   ON CONFLICT (ieee, endpoint_id, cluster, attrid)
+        q = f"""INSERT INTO unsupported_attributes{DB_V} VALUES (?, ?, ?, ?, ?)
+                   ON CONFLICT (ieee, endpoint_id, cluster_type, cluster_id, attr_id)
                    DO NOTHING"""
-        await self.execute(q, (ieee, endpoint_id, cluster_id, attrid))
+        await self.execute(q, (ieee, endpoint_id, cluster_type, cluster_id, attrid))
         await self._db.commit()
 
     def unsupported_attribute_removed(
@@ -311,18 +338,25 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             "_unsupported_attribute_removed",
             cluster.endpoint.device.ieee,
             cluster.endpoint.endpoint_id,
+            cluster.cluster_type,
             cluster.cluster_id,
             attrid,
         )
 
     async def _unsupported_attribute_removed(
-        self, ieee: t.EUI64, endpoint_id: int, cluster_id: int, attrid: int
+        self,
+        ieee: t.EUI64,
+        endpoint_id: int,
+        cluster_type: ClusterType,
+        cluster_id: int,
+        attrid: int,
     ) -> None:
         q = f"""DELETE FROM unsupported_attributes{DB_V} WHERE ieee = ?
                                                          AND endpoint_id = ?
-                                                         AND cluster = ?
-                                                         AND attrid = ?"""
-        await self.execute(q, (ieee, endpoint_id, cluster_id, attrid))
+                                                         AND cluster_type = ?
+                                                         AND cluster_id = ?
+                                                         AND attr_id = ?"""
+        await self.execute(q, (ieee, endpoint_id, cluster_type, cluster_id, attrid))
         await self._db.commit()
 
     def neighbors_updated(self, ieee: t.EUI64, neighbors: list[zdo_t.Neighbor]) -> None:
@@ -334,7 +368,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     ) -> None:
         await self.execute(f"DELETE FROM neighbors{DB_V} WHERE device_ieee = ?", [ieee])
 
-        rows = [(ieee,) + neighbor.as_tuple() for neighbor in neighbors]
+        rows = [(ieee, *neighbor.as_tuple()) for neighbor in neighbors]
 
         await self._db.executemany(
             f"INSERT INTO neighbors{DB_V} VALUES (?,?,?,?,?,?,?,?,?,?,?,?)", rows
@@ -348,7 +382,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     async def _routes_updated(self, ieee: t.EUI64, routes: list[zdo_t.Route]) -> None:
         await self.execute(f"DELETE FROM routes{DB_V} WHERE device_ieee = ?", [ieee])
 
-        rows = [(ieee,) + route.as_tuple() for route in routes]
+        rows = [(ieee, *route.as_tuple()) for route in routes]
 
         await self._db.executemany(
             f"INSERT INTO routes{DB_V} VALUES (?,?,?,?,?,?,?,?)", rows
@@ -436,16 +470,15 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         if device.node_desc is not None:
             await self._save_node_descriptor(device)
 
-        if isinstance(device, zigpy.quirks.CustomDevice):
+        if isinstance(device, zigpy.quirks.BaseCustomDevice):
             await self._db.commit()
             return
 
         await self._save_endpoints(device)
         for ep in device.non_zdo_endpoints:
-            await self._save_input_clusters(ep)
+            await self._save_clusters(ep)
             await self._save_attribute_cache(ep)
             await self._save_unsupported_attributes(ep)
-            await self._save_output_clusters(ep)
         await self._db.commit()
 
     async def _save_endpoints(self, device: zigpy.typing.DeviceType) -> None:
@@ -488,15 +521,20 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 maximum_outgoing_transfer_size=excluded.maximum_outgoing_transfer_size,
                 descriptor_capability_field=excluded.descriptor_capability_field"""
 
-        await self.execute(q, (device.ieee,) + device.node_desc.as_tuple())
+        await self.execute(q, (device.ieee, *device.node_desc.as_tuple()))
 
-    async def _save_input_clusters(self, endpoint: zigpy.typing.EndpointType) -> None:
+    async def _save_clusters(self, endpoint: zigpy.typing.EndpointType) -> None:
         clusters = [
-            (endpoint.device.ieee, endpoint.endpoint_id, cluster.cluster_id)
-            for cluster in endpoint.in_clusters.values()
+            (
+                endpoint.device.ieee,
+                endpoint.endpoint_id,
+                cluster.cluster_type,
+                cluster.cluster_id,
+            )
+            for cluster in endpoint.clusters
         ]
-        q = f"""INSERT INTO in_clusters{DB_V} VALUES (?, ?, ?)
-                    ON CONFLICT (ieee, endpoint_id, cluster)
+        q = f"""INSERT INTO clusters{DB_V} VALUES (?, ?, ?, ?)
+                    ON CONFLICT (ieee, endpoint_id, cluster_type, cluster_id)
                     DO NOTHING"""
         await self._db.executemany(q, clusters)
 
@@ -505,38 +543,35 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             (
                 ep.device.ieee,
                 ep.endpoint_id,
+                cluster.cluster_type,
                 cluster.cluster_id,
                 attrid,
                 value,
                 cluster._attr_last_updated.get(attrid, UNIX_EPOCH).timestamp(),
             )
-            for cluster in ep.in_clusters.values()
+            for cluster in ep.clusters
             for attrid, value in cluster._attr_cache.items()
         ]
-        q = f"""INSERT INTO attributes_cache{DB_V} VALUES (?, ?, ?, ?, ?, ?)
-                    ON CONFLICT (ieee, endpoint_id, cluster, attrid)
+        q = f"""INSERT INTO attributes_cache{DB_V} VALUES (?, ?, ?, ?, ?, ?, ?)
+                    ON CONFLICT (ieee, endpoint_id, cluster_type, cluster_id, attr_id)
                     DO UPDATE SET value=excluded.value, last_updated=excluded.last_updated"""
         await self._db.executemany(q, clusters)
 
     async def _save_unsupported_attributes(self, ep: zigpy.typing.EndpointType) -> None:
         clusters = [
-            (ep.device.ieee, ep.endpoint_id, cluster.cluster_id, attr)
-            for cluster in ep.in_clusters.values()
+            (
+                ep.device.ieee,
+                ep.endpoint_id,
+                cluster.cluster_type,
+                cluster.cluster_id,
+                attr,
+            )
+            for cluster in ep.clusters
             for attr in cluster.unsupported_attributes
             if isinstance(attr, int)
         ]
-        q = f"""INSERT INTO unsupported_attributes{DB_V} VALUES (?, ?, ?, ?)
-                    ON CONFLICT (ieee, endpoint_id, cluster, attrid)
-                    DO NOTHING"""
-        await self._db.executemany(q, clusters)
-
-    async def _save_output_clusters(self, endpoint: zigpy.typing.EndpointType) -> None:
-        clusters = [
-            (endpoint.device.ieee, endpoint.endpoint_id, cluster.cluster_id)
-            for cluster in endpoint.out_clusters.values()
-        ]
-        q = f"""INSERT INTO out_clusters{DB_V} VALUES (?, ?, ?)
-                    ON CONFLICT (ieee, endpoint_id, cluster)
+        q = f"""INSERT INTO unsupported_attributes{DB_V} VALUES (?, ?, ?, ?, ?)
+                    ON CONFLICT (ieee, endpoint_id, cluster_type, cluster_id, attr_id)
                     DO NOTHING"""
         await self._db.executemany(q, clusters)
 
@@ -544,6 +579,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         self,
         ieee: t.EUI64,
         endpoint_id: int,
+        cluster_type: ClusterType,
         cluster_id: int,
         attrid: int,
         value: Any,
@@ -551,8 +587,8 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     ) -> None:
         q = f"""
             INSERT INTO attributes_cache{DB_V}
-            VALUES (:ieee, :endpoint_id, :cluster_id, :attrid, :value, :timestamp)
-                ON CONFLICT (ieee, endpoint_id, cluster, attrid) DO UPDATE
+            VALUES (:ieee, :endpoint_id, :cluster_type, :cluster_id, :attr_id, :value, :timestamp)
+                ON CONFLICT (ieee, endpoint_id, cluster_type, cluster_id, attr_id) DO UPDATE
                 SET value=excluded.value, last_updated=excluded.last_updated
                 WHERE
                     value != excluded.value
@@ -563,8 +599,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             {
                 "ieee": ieee,
                 "endpoint_id": endpoint_id,
+                "cluster_type": cluster_type,
                 "cluster_id": cluster_id,
-                "attrid": attrid,
+                "attr_id": attrid,
                 "value": value,
                 "timestamp": timestamp.timestamp(),
                 "min_update_delta": MIN_UPDATE_DELTA,
@@ -572,6 +609,36 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         )
         await self._db.commit()
 
+    async def _clear_attribute(
+        self,
+        ieee: t.EUI64,
+        endpoint_id: int,
+        cluster_type: ClusterType,
+        cluster_id: int,
+        attrid: int,
+    ) -> None:
+        q = f"""
+            DELETE FROM attributes_cache{DB_V}
+            WHERE
+                ieee = :ieee
+                AND endpoint_id = :endpoint_id
+                AND cluster_type = :cluster_type
+                AND cluster_id = :cluster_id
+                AND attr_id = :attr_id
+            """
+
+        await self.execute(
+            q,
+            {
+                "ieee": ieee,
+                "endpoint_id": endpoint_id,
+                "cluster_type": cluster_type,
+                "cluster_id": cluster_id,
+                "attr_id": attrid,
+            },
+        )
+        await self._db.commit()
+
     def network_backup_created(self, backup: zigpy.backups.NetworkBackup) -> None:
         self.enqueue("_network_backup_created", json.dumps(backup.as_dict()))
 
@@ -602,7 +669,16 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         await self._load_clusters()
 
         # Quirks require the manufacturer and model name to be populated
-        await self._load_attributes("attrid=4 OR attrid=5")
+        await self._load_attributes(
+            f"""
+                cluster_type={ClusterType.Server}
+            AND cluster_id={Basic.cluster_id}
+            AND (
+                   attr_id={Basic.AttributeDefs.manufacturer.id}
+                OR attr_id={Basic.AttributeDefs.model.id}
+            )
+            """
+        )
 
         for device in self._application.devices.values():
             device = zigpy.quirks.get_device(device)
@@ -618,7 +694,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         await self._load_network_backups()
         await self._register_device_listeners()
 
-    async def _load_attributes(self, filter: str = None) -> None:
+    async def _load_attributes(self, filter: str | None = None) -> None:
         if filter:
             query = f"SELECT * FROM attributes_cache{DB_V} WHERE {filter}"
         else:
@@ -628,8 +704,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
             async for (
                 ieee,
                 endpoint_id,
-                cluster,
-                attrid,
+                cluster_type,
+                cluster_id,
+                attr_id,
                 value,
                 last_updated,
             ) in cursor:
@@ -640,28 +717,39 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                     continue
 
                 ep = dev.endpoints[endpoint_id]
+                clusters = (
+                    ep.in_clusters
+                    if cluster_type == ClusterType.Server
+                    else ep.out_clusters
+                )
 
-                if cluster not in ep.in_clusters:
+                if cluster_id not in clusters:
                     continue
 
-                ep.in_clusters[cluster]._attr_cache[attrid] = value
-                ep.in_clusters[cluster]._attr_last_updated[
-                    attrid
-                ] = datetime.fromtimestamp(last_updated, timezone.utc)
+                clusters[cluster_id]._attr_cache[attr_id] = value
+                clusters[cluster_id]._attr_last_updated[attr_id] = (
+                    datetime.fromtimestamp(last_updated, UTC)
+                )
 
                 LOGGER.debug(
                     "[0x%04x:%s:0x%04x] Attribute id: %s value: %s",
                     dev.nwk,
                     endpoint_id,
-                    cluster,
-                    attrid,
+                    cluster_id,
+                    attr_id,
                     value,
                 )
 
                 # Populate the device's manufacturer and model attributes
-                if cluster == Basic.cluster_id and attrid == 0x0004:
+                if (
+                    cluster_id == Basic.cluster_id
+                    and attr_id == Basic.AttributeDefs.manufacturer.id
+                ):
                     dev.manufacturer = decode_str_attribute(value)
-                elif cluster == Basic.cluster_id and attrid == 0x0005:
+                elif (
+                    cluster_id == Basic.cluster_id
+                    and attr_id == Basic.AttributeDefs.model.id
+                ):
                     dev.model = decode_str_attribute(value)
 
     async def _load_unsupported_attributes(self) -> None:
@@ -670,7 +758,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         async with self.execute(
             f"SELECT * FROM unsupported_attributes{DB_V}"
         ) as cursor:
-            async for (ieee, endpoint_id, cluster_id, attrid) in cursor:
+            async for ieee, endpoint_id, cluster_type, cluster_id, attr_id in cursor:
                 dev = self._application.get_device(ieee)
 
                 try:
@@ -678,16 +766,22 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 except KeyError:
                     continue
 
+                clusters = (
+                    ep.in_clusters
+                    if cluster_type == ClusterType.Server
+                    else ep.out_clusters
+                )
+
                 try:
-                    cluster = ep.in_clusters[cluster_id]
+                    cluster = clusters[cluster_id]
                 except KeyError:
                     continue
 
-                cluster.add_unsupported_attribute(attrid, inhibit_events=True)
+                cluster.add_unsupported_attribute(attr_id, inhibit_events=True)
 
     async def _load_devices(self) -> None:
         async with self.execute(f"SELECT * FROM devices{DB_V}") as cursor:
-            async for (ieee, nwk, status, last_seen) in cursor:
+            async for ieee, nwk, status, last_seen in cursor:
                 dev = self._application.add_device(ieee, nwk)
                 dev.status = zigpy.device.Status(status)
 
@@ -696,14 +790,14 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
 
     async def _load_node_descriptors(self) -> None:
         async with self.execute(f"SELECT * FROM node_descriptors{DB_V}") as cursor:
-            async for (ieee, *fields) in cursor:
+            async for ieee, *fields in cursor:
                 dev = self._application.get_device(ieee)
                 dev.node_desc = zdo_t.NodeDescriptor(*fields)
                 assert dev.node_desc.is_valid
 
     async def _load_endpoints(self) -> None:
         async with self.execute(f"SELECT * FROM endpoints{DB_V}") as cursor:
-            async for (ieee, epid, profile_id, device_type, status) in cursor:
+            async for ieee, epid, profile_id, device_type, status in cursor:
                 dev = self._application.get_device(ieee)
                 ep = dev.add_endpoint(epid)
                 ep.profile_id = profile_id
@@ -717,35 +811,34 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                     ep.device_type = device_type
 
     async def _load_clusters(self) -> None:
-        async with self.execute(f"SELECT * FROM in_clusters{DB_V}") as cursor:
-            async for (ieee, endpoint_id, cluster) in cursor:
+        async with self.execute(f"SELECT * FROM clusters{DB_V}") as cursor:
+            async for ieee, endpoint_id, cluster_type, cluster_id in cursor:
                 dev = self._application.get_device(ieee)
                 ep = dev.endpoints[endpoint_id]
-                ep.add_input_cluster(cluster)
 
-        async with self.execute(f"SELECT * FROM out_clusters{DB_V}") as cursor:
-            async for (ieee, endpoint_id, cluster) in cursor:
-                dev = self._application.get_device(ieee)
-                ep = dev.endpoints[endpoint_id]
-                ep.add_output_cluster(cluster)
+                if ClusterType(cluster_type) == ClusterType.Server:
+                    ep.add_input_cluster(cluster_id)
+                else:
+                    ep.add_output_cluster(cluster_id)
 
     async def _load_groups(self) -> None:
         async with self.execute(f"SELECT * FROM groups{DB_V}") as cursor:
-            async for (group_id, name) in cursor:
+            async for group_id, name in cursor:
                 self._application.groups.add_group(group_id, name, suppress_event=True)
 
     async def _load_group_members(self) -> None:
         async with self.execute(f"SELECT * FROM group_members{DB_V}") as cursor:
-            async for (group_id, ieee, ep_id) in cursor:
+            async for group_id, ieee, ep_id in cursor:
                 dev = self._application.get_device(ieee)
                 group = self._application.groups[group_id]
                 group.add_member(dev.endpoints[ep_id], suppress_event=True)
 
     async def _load_relays(self) -> None:
         async with self.execute(f"SELECT * FROM relays{DB_V}") as cursor:
-            async for (ieee, value) in cursor:
+            async for ieee, value in cursor:
                 dev = self._application.get_device(ieee)
-                dev.relays, _ = t.Relays.deserialize(value)
+                relays, _ = t.Relays.deserialize(value)
+                dev.relays = zigpy.util.filter_relays(relays)
 
     async def _load_neighbors(self) -> None:
         async with self.execute(f"SELECT * FROM neighbors{DB_V}") as cursor:
@@ -786,7 +879,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
 
         try:
             yield
-        except Exception:
+        except Exception:  # noqa: BLE001
             await self.execute("ROLLBACK")
             raise
         else:
@@ -865,6 +958,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 (self._migrate_to_v10, 10),
                 (self._migrate_to_v11, 11),
                 (self._migrate_to_v12, 12),
+                (self._migrate_to_v13, 13),
             ]:
                 if db_version >= min(to_db_version, DB_VERSION):
                     continue
@@ -923,7 +1017,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                         elif errors == "ignore":
                             pass
                         else:
-                            raise ValueError(f"Invalid value for `errors`: {errors!r}")
+                            raise ValueError(
+                                f"Invalid value for `errors`: {errors!r}"
+                            ) from e
 
     async def _migrate_to_v4(self):
         """Schema v4 expanded the node descriptor and neighbor table columns"""
@@ -937,7 +1033,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                     await self.execute(
                         "INSERT INTO node_descriptors_v4"
                         " VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
-                        (dev_ieee,) + node_desc.as_tuple(),
+                        (dev_ieee, *node_desc.as_tuple()),
                     )
 
         # The `neighbors` table was added in v3 but the version number was not
@@ -958,7 +1054,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
 
                     await self.execute(
                         "INSERT INTO neighbors_v4 VALUES (?,?,?,?,?,?,?,?,?,?,?,?)",
-                        (dev_ieee,) + neighbor.as_tuple(),
+                        (dev_ieee, *neighbor.as_tuple()),
                     )
 
     async def _migrate_to_v5(self):
@@ -1054,7 +1150,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         """Schema v8 added the `devices_v8.last_seen` column."""
 
         async with self.execute("SELECT * FROM devices_v7") as cursor:
-            async for (ieee, nwk, status) in cursor:
+            async for ieee, nwk, status in cursor:
                 # Set the default `last_seen` to the unix epoch
                 await self.execute(
                     "INSERT INTO devices_v8 VALUES (?, ?, ?, ?)",
@@ -1163,9 +1259,73 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         )
 
         async with self.execute("SELECT * FROM attributes_cache_v11") as cursor:
-            async for (ieee, endpoint_id, cluster_id, attrid, value) in cursor:
+            async for ieee, endpoint_id, cluster_id, attrid, value in cursor:
                 # Set the default `last_updated` to the unix epoch
                 await self.execute(
                     "INSERT INTO attributes_cache_v12 VALUES (?, ?, ?, ?, ?, ?)",
                     (ieee, endpoint_id, cluster_id, attrid, value, 0),
                 )
+
+    async def _migrate_to_v13(self):
+        """Schema v13 combines both cluster types and caching for all attributes."""
+
+        await self._migrate_tables(
+            {
+                "devices_v12": "devices_v13",
+                "endpoints_v12": "endpoints_v13",
+                "neighbors_v12": "neighbors_v13",
+                "routes_v12": "routes_v13",
+                "node_descriptors_v12": "node_descriptors_v13",
+                "groups_v12": "groups_v13",
+                "group_members_v12": "group_members_v13",
+                "relays_v12": "relays_v13",
+                "network_backups_v12": "network_backups_v13",
+                "in_clusters_v12": None,
+                "out_clusters_v12": None,
+                "unsupported_attributes_v12": None,
+                "attributes_cache_v12": None,
+            }
+        )
+
+        async with self.execute("SELECT * FROM in_clusters_v12") as cursor:
+            async for ieee, endpoint_id, cluster_id in cursor:
+                await self.execute(
+                    "INSERT INTO clusters_v13 VALUES (?, ?, ?, ?)",
+                    (ieee, endpoint_id, ClusterType.Server, cluster_id),
+                )
+
+        async with self.execute("SELECT * FROM out_clusters_v12") as cursor:
+            async for ieee, endpoint_id, cluster_id in cursor:
+                await self.execute(
+                    "INSERT INTO clusters_v13 VALUES (?, ?, ?, ?)",
+                    (ieee, endpoint_id, ClusterType.Client, cluster_id),
+                )
+
+        async with self.execute("SELECT * FROM unsupported_attributes_v12") as cursor:
+            async for ieee, endpoint_id, cluster_id, attrid in cursor:
+                await self.execute(
+                    "INSERT INTO unsupported_attributes_v13 VALUES (?, ?, ?, ?, ?)",
+                    (ieee, endpoint_id, ClusterType.Server, cluster_id, attrid),
+                )
+
+        async with self.execute("SELECT * FROM attributes_cache_v12") as cursor:
+            async for (
+                ieee,
+                endpoint_id,
+                cluster_id,
+                attrid,
+                value,
+                last_updated,
+            ) in cursor:
+                await self.execute(
+                    "INSERT INTO attributes_cache_v13 VALUES (?, ?, ?, ?, ?, ?, ?)",
+                    (
+                        ieee,
+                        endpoint_id,
+                        ClusterType.Server,
+                        cluster_id,
+                        attrid,
+                        value,
+                        last_updated,
+                    ),
+                )
