@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import asyncio
 import enum
 import logging
 from typing import Any
@@ -9,16 +8,10 @@ from zigpy.const import APS_REPLY_TIMEOUT
 import zigpy.exceptions
 import zigpy.profiles
 import zigpy.types as t
-from zigpy.typing import AddressingMode, DeviceType
+from zigpy.typing import DeviceType
 import zigpy.util
 import zigpy.zcl
-from zigpy.zcl.foundation import (
-    GENERAL_COMMANDS,
-    CommandSchema,
-    GeneralCommand,
-    Status as ZCLStatus,
-    ZCLHeader,
-)
+from zigpy.zcl.foundation import GENERAL_COMMANDS, GeneralCommand, Status as ZCLStatus
 from zigpy.zdo.types import Status as ZDOStatus
 
 LOGGER = logging.getLogger(__name__)
@@ -62,7 +55,7 @@ class Endpoint(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
             self.info("Endpoint descriptor already queried")
         else:
             status, _, sd = await self._device.zdo.Simple_Desc_req(
-                self._device.nwk, self._endpoint_id, priority=t.PacketPriority.HIGH
+                self._device.nwk, self._endpoint_id
             )
 
             if status == ZDOStatus.NOT_ACTIVE:
@@ -181,7 +174,7 @@ class Endpoint(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
             res = await self.groups.get_membership(groups=[])
         except AttributeError:
             return
-        except (asyncio.TimeoutError, zigpy.exceptions.ZigbeeException):
+        except (TimeoutError, zigpy.exceptions.ZigbeeException):
             self.debug("Failed to sync-up group membership")
             return
 
@@ -200,9 +193,9 @@ class Endpoint(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         for names in (["manufacturer", "model"], ["manufacturer"], ["model"]):
             try:
                 success, failure = await self.basic.read_attributes(
-                    names, allow_cache=True, priority=t.PacketPriority.HIGH
+                    names, allow_cache=True
                 )
-            except asyncio.TimeoutError:
+            except TimeoutError:
                 # Only swallow the `TimeoutError` on the double attribute read
                 if len(names) == 2:
                     continue
@@ -217,36 +210,6 @@ class Endpoint(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
 
         return self._model, self._manufacturer
 
-    def deserialize(
-        self, cluster_id: t.ClusterId, data: bytes
-    ) -> tuple[ZCLHeader, CommandSchema]:
-        """Deserialize data for ZCL"""
-        if cluster_id not in self.in_clusters and cluster_id not in self.out_clusters:
-            raise KeyError(f"No cluster ID 0x{cluster_id:04x} on {self.unique_id}")
-
-        cluster = self.in_clusters.get(cluster_id, self.out_clusters.get(cluster_id))
-        return cluster.deserialize(data)
-
-    def handle_message(
-        self,
-        profile: int,
-        cluster: int,
-        hdr: ZCLHeader,
-        args: list,
-        *,
-        dst_addressing: AddressingMode | None = None,
-    ) -> None:
-        if cluster in self.in_clusters:
-            handler = self.in_clusters[cluster].handle_message
-        elif cluster in self.out_clusters:
-            handler = self.out_clusters[cluster].handle_message
-        else:
-            self.debug("Message on unknown cluster 0x%04x", cluster)
-            self.listener_event("unknown_cluster_message", hdr.command_id, args)
-            return
-
-        handler(hdr, args, dst_addressing=dst_addressing)
-
     async def request(
         self,
         cluster: t.ClusterId,
@@ -257,7 +220,7 @@ class Endpoint(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         expect_reply: bool = True,
         use_ieee: bool = False,
         ask_for_ack: bool | None = None,
-        priority: int = t.PacketPriority.NORMAL,
+        priority: int | None = None,
     ):
         if self.profile_id == zigpy.profiles.zll.PROFILE_ID and not (
             cluster == zigpy.zcl.clusters.lightlink.LightLink.cluster_id
@@ -291,7 +254,7 @@ class Endpoint(zigpy.util.LocalLogMixin, zigpy.util.ListenableMixin):
         expect_reply: bool = False,
         use_ieee: bool = False,
         ask_for_ack: bool | None = None,
-        priority: int = t.PacketPriority.NORMAL,
+        priority: int | None = None,
     ) -> None:
         if self.profile_id == zigpy.profiles.zll.PROFILE_ID and not (
             cluster == zigpy.zcl.clusters.lightlink.LightLink.cluster_id
