@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import collections
 from collections.abc import Iterable, Sequence
-from datetime import datetime, timezone
+from datetime import UTC, datetime
 import enum
 import functools
 import itertools
@@ -199,7 +199,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
 
                 if isinstance(
                     definition,
-                    (foundation.ZCLCommandDef, foundation.ZCLAttributeDef),
+                    foundation.ZCLCommandDef | foundation.ZCLAttributeDef,
                 ):
                     if definition.name is None:
                         object.__setattr__(definition, "name", name)
@@ -288,7 +288,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         return cluster
 
     def deserialize(self, data: bytes) -> tuple[foundation.ZCLHeader, ...]:
-        self.debug("Received ZCL frame: %r", data)
+        self.debug("Received ZCL frame: %r", data.hex(" "))
 
         hdr, data = foundation.ZCLHeader.deserialize(data)
         self.debug("Decoded ZCL frame header: %r", hdr)
@@ -301,14 +301,18 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
                 commands = self.server_commands
 
             if hdr.command_id not in commands:
-                self.debug("Unknown cluster command %s %s", hdr.command_id, data)
+                self.debug(
+                    "Unknown cluster command %s %r", hdr.command_id, data.hex(" ")
+                )
                 return hdr, data
 
             command = commands[hdr.command_id]
         else:
             # General command
             if hdr.command_id not in foundation.GENERAL_COMMANDS:
-                self.debug("Unknown foundation command %s %s", hdr.command_id, data)
+                self.debug(
+                    "Unknown foundation command %s %r", hdr.command_id, data.hex(" ")
+                )
                 return hdr, data
 
             command = foundation.GENERAL_COMMANDS[hdr.command_id]
@@ -318,7 +322,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         self.debug("Decoded ZCL frame: %s:%r", type(self).__name__, response)
 
         if data:
-            self.debug("Data remains after deserializing ZCL frame: %r", data)
+            self.debug("Data remains after deserializing ZCL frame: %r", data.hex(" "))
 
         return hdr, response
 
@@ -738,7 +742,11 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         cfg = foundation.AttributeReportingConfig()
         cfg.direction = direction
         cfg.attrid = attr_def.id
-        cfg.datatype = foundation.DataType.from_python_type(attr_def.type).type_id
+        cfg.datatype = (
+            attr_def.zcl_type
+            if attr_def.zcl_type is not None
+            else foundation.DataType.from_python_type(attr_def.type).type_id
+        )
         cfg.min_interval = min_interval
         cfg.max_interval = max_interval
         cfg.reportable_change = reportable_change
@@ -893,7 +901,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
             self._attr_last_updated.pop(attrid)
             self.listener_event("attribute_cleared", attrid)
         else:
-            now = datetime.now(timezone.utc)
+            now = datetime.now(UTC)
             self._attr_cache[attrid] = value
             self._attr_last_updated[attrid] = now
             self.listener_event("attribute_updated", attrid, value, now)
@@ -936,7 +944,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
 
     def __setitem__(self, key: int | str, value: Any) -> None:
         """Set cached value through attribute write."""
-        if not isinstance(key, (int, str)):
+        if not isinstance(key, int | str):
             raise ValueError("attr_name or attr_id are accepted only")  # noqa: TRY004
         self.create_catching_task(self.write_attributes({key: value}))
 
